@@ -220,6 +220,7 @@ static void
 gst_hls_demux2_init (GstHLSDemux * demux)
 {
   demux->keys = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+  demux->start_bitrate = DEFAULT_START_BITRATE;
   g_mutex_init (&demux->keys_lock);
 }
 
@@ -296,11 +297,17 @@ gst_hls_demux_wait_for_variant_playlist (GstHLSDemux * hlsdemux)
 
   while ((flow_ret = gst_hls_demux_check_variant_playlist_loaded (hlsdemux)
           == GST_ADAPTIVE_DEMUX_FLOW_BUSY)) {
-    if (!gst_adaptive_demux2_stream_wait_prepared (GST_ADAPTIVE_DEMUX2_STREAM
-            (hlsdemux->main_stream))) {
-      GST_DEBUG_OBJECT (hlsdemux,
-          "Interrupted waiting for stream to be prepared");
-      return GST_FLOW_FLUSHING;
+    GstAdaptiveDemux2Stream *stream =
+        GST_ADAPTIVE_DEMUX2_STREAM (hlsdemux->main_stream);
+
+    if (stream->state != GST_ADAPTIVE_DEMUX2_STREAM_STATE_STOPPED) {
+      stream->state = GST_ADAPTIVE_DEMUX2_STREAM_STATE_WAITING_PREPARE;
+
+      if (!gst_adaptive_demux2_stream_wait_prepared (stream)) {
+        GST_DEBUG_OBJECT (hlsdemux,
+            "Interrupted waiting for stream to be prepared");
+        return GST_FLOW_FLUSHING;
+      }
     }
   }
 
@@ -451,7 +458,9 @@ create_main_variant_stream (GstHLSDemux * demux)
 
   gst_hls_demux_stream_set_playlist_uri (hlsdemux_stream,
       demux->current_variant->uri);
-  gst_hls_demux_stream_start_playlist_loading (hlsdemux_stream);
+
+  /* Start this stream, which will trigger a playlist load */
+  gst_adaptive_demux2_stream_start (stream);
 }
 
 GstAdaptiveDemuxTrack *
@@ -550,7 +559,8 @@ existing_rendition_stream (GList * streams, GstHLSRenditionStream * media)
     if (demux_stream->rendition_type == stream_type) {
       if (!g_strcmp0 (demux_stream->name, media->name))
         return demux_stream;
-      if (media->lang && !g_strcmp0 (demux_stream->lang, media->lang))
+      if (media->name == NULL && media->lang
+          && !g_strcmp0 (demux_stream->lang, media->lang))
         return demux_stream;
     }
   }
@@ -916,6 +926,34 @@ gst_hls_time_map_free (GstHLSTimeMap * map)
   g_free (map);
 }
 
+void
+gst_time_map_set_values (GstHLSTimeMap * map, GstClockTimeDiff stream_time,
+    GstClockTime internal_time, GDateTime * pdt)
+{
+  GstClockTime offset = 0;
+
+  if (stream_time < 0) {
+    offset = -stream_time;
+    stream_time = 0;
+    /* Handle negative stream times. This can happen for example when the server
+     * returns an older playlist.
+     *
+     * Shift the values accordingly to end up with non-negative reference stream
+     * time */
+    GST_DEBUG ("Shifting values before storage (offset : %" GST_TIME_FORMAT ")",
+        GST_TIME_ARGS (offset));
+  }
+
+  map->stream_time = stream_time;
+  map->internal_time = internal_time;
+  if (pdt) {
+    if (offset)
+      map->pdt = g_date_time_add (pdt, offset / GST_USECOND);
+    else
+      map->pdt = g_date_time_ref (pdt);
+  }
+}
+
 void
 gst_hls_demux_add_time_mapping (GstHLSDemux * demux, gint64 dsn,
     GstClockTimeDiff stream_time, GDateTime * pdt)
@@ -925,7 +963,6 @@ gst_hls_demux_add_time_mapping (GstHLSDemux * demux, gint64 dsn,
 #endif
   GstHLSTimeMap *map;
   GList *tmp;
-  GstClockTime offset = 0;
 
   /* Check if we don't already have a mapping for the given dsn */
   for (tmp = demux->mappings; tmp; tmp = tmp->next) {
@@ -955,28 +992,9 @@ gst_hls_demux_add_time_mapping (GstHLSDemux * demux, gint64 dsn,
   g_free (datestring);
 #endif
 
-  if (stream_time < 0) {
-    offset = -stream_time;
-    stream_time = 0;
-    /* Handle negative stream times. This can happen for example when the server
-     * returns an older playlist.
-     *
-     * Shift the values accordingly to end up with non-negative reference stream
-     * time */
-    GST_DEBUG_OBJECT (demux,
-        "Shifting values before storage (offset : %" GST_TIME_FORMAT ")",
-        GST_TIME_ARGS (offset));
-  }
-
   map = gst_hls_time_map_new ();
   map->dsn = dsn;
-  map->stream_time = stream_time;
-  if (pdt) {
-    if (offset)
-      map->pdt = g_date_time_add (pdt, offset / GST_USECOND);
-    else
-      map->pdt = g_date_time_ref (pdt);
-  }
+  gst_time_map_set_values (map, stream_time, GST_CLOCK_TIME_NONE, pdt);
 
   demux->mappings = g_list_append (demux->mappings, map);
 }
@@ -1108,8 +1126,7 @@ gst_hls_demux_handle_variant_playlist_update (GstHLSDemux * demux,
                   main_uri, "uri", G_TYPE_STRING,
                   uri, "bitrate", G_TYPE_INT, new_bandwidth, NULL)));
 
-      /* Mark discont on the next packet after switching variant */
-      GST_ADAPTIVE_DEMUX2_STREAM (demux->main_stream)->discont = TRUE;
+      GST_DEBUG_OBJECT (demux, "Changed variant");
     }
   }
 
@@ -1223,8 +1240,8 @@ gst_hls_demux_reset_for_lost_sync (GstHLSDemux * hlsdemux)
         gst_hls_media_playlist_recalculate_stream_time (hls_stream->playlist,
             hls_stream->current_segment);
         GST_DEBUG_OBJECT (stream,
-            "Resynced variant playlist to %" GST_STIME_FORMAT,
-            GST_STIME_ARGS (stream->current_position));
+            "Resynced variant playlist to %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (stream->current_position));
         map =
             gst_hls_demux_find_time_map (hlsdemux,
             hls_stream->current_segment->discont_sequence);
