@@ -21,11 +21,12 @@
 
 /**
  * SECTION:element-id3demux
+ * @title: id3demux
  *
  * id3demux accepts data streams with either (or both) ID3v2 regions at the
  * start, or ID3v1 at the end. The mime type of the data between the tag blocks
  * is detected using typefind functions, and the appropriate output mime type
- * set on outgoing buffers. 
+ * set on outgoing buffers.
  *
  * The element is only able to read ID3v1 tags from a seekable stream, because
  * they are at the end of the stream. That is, when get_range mode is supported
@@ -36,25 +37,25 @@
  * This id3demux element replaced an older element with the same name which
  * relied on libid3tag from the MAD project.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 filesrc location=file.mp3 ! id3demux ! fakesink -t
  * ]| This pipeline should read any available ID3 tag information and output it.
  * The contents of the file inside the ID3 tag regions should be detected, and
  * the appropriate mime type set on buffers produced from id3demux.
- * </refsect2>
+ *
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 #include <gst/gst.h>
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 #include <gst/tag/tag.h>
 #include <gst/pbutils/pbutils.h>
 #include <string.h>
 
 #include "gstid3demux.h"
+#include "gstid3metaparse.h"
 
 enum
 {
@@ -91,6 +92,12 @@ static void gst_id3demux_get_property (GObject * object, guint prop_id,
 
 #define gst_id3demux_parent_class parent_class
 G_DEFINE_TYPE (GstID3Demux, gst_id3demux, GST_TYPE_TAG_DEMUX);
+#define _do_init \
+  GST_DEBUG_CATEGORY_INIT (id3demux_debug, "id3demux", 0, \
+      "GStreamer ID3 tag demuxer"); \
+  gst_tag_register_musicbrainz_tags ();
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (id3demux, "id3demux",
+    GST_RANK_PRIMARY, GST_TYPE_ID3DEMUX, _do_init);
 
 static void
 gst_id3demux_class_init (GstID3DemuxClass * klass)
@@ -273,13 +280,11 @@ gst_id3demux_get_property (GObject * object, guint prop_id,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  GST_DEBUG_CATEGORY_INIT (id3demux_debug, "id3demux", 0,
-      "GStreamer ID3 tag demuxer");
+  gboolean res = GST_ELEMENT_REGISTER (id3metaparse, plugin);
 
-  gst_tag_register_musicbrainz_tags ();
+  res |= GST_ELEMENT_REGISTER (id3demux, plugin);
 
-  return gst_element_register (plugin, "id3demux",
-      GST_RANK_PRIMARY, GST_TYPE_ID3DEMUX);
+  return res;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
