@@ -41,6 +41,7 @@
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/video/video.h>
 
+#include "gstrtpelements.h"
 #include "gstrtpjpegpay.h"
 #include "gstrtputils.h"
 #include "gstbuffermemory.h"
@@ -193,7 +194,7 @@ typedef struct
 typedef struct
 {
   guint8 size;
-  const guint8 *data;
+  guint offset;
 } RtpQuantTable;
 
 /*
@@ -248,6 +249,8 @@ static GstFlowReturn gst_rtp_jpeg_pay_handle_buffer (GstRTPBasePayload * pad,
 
 #define gst_rtp_jpeg_pay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpJPEGPay, gst_rtp_jpeg_pay, GST_TYPE_RTP_BASE_PAYLOAD);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpjpegpay, "rtpjpegpay",
+    GST_RANK_SECONDARY, GST_TYPE_RTP_JPEG_PAY, rtp_element_init (plugin));
 
 static void
 gst_rtp_jpeg_pay_class_init (GstRtpJPEGPayClass * klass)
@@ -466,7 +469,7 @@ gst_rtp_jpeg_pay_read_quant_table (GstBufferMemoryMap * memory,
     GST_LOG ("read quant table %d, tab_size %d, prec %02x", id, tab_size, prec);
 
     tables[id].size = tab_size;
-    tables[id].data = memory->data;
+    tables[id].offset = memory->offset;
 
     quant_size -= (tab_size + 1);
     if (!gst_buffer_memory_advance_bytes (memory, tab_size)) {
@@ -718,7 +721,7 @@ gst_rtp_jpeg_pay_handle_buffer (GstRTPBasePayload * basepayload,
   RtpJpegHeader jpeg_header;
   RtpQuantHeader quant_header;
   RtpRestartMarkerHeader restart_marker_header;
-  RtpQuantTable tables[15] = { {0, NULL}, };
+  RtpQuantTable tables[15] = { {0, 0}, };
   CompInfo info[3] = { {0,}, };
   guint quant_data_size;
   guint mtu, max_payload_size;
@@ -777,8 +780,9 @@ gst_rtp_jpeg_pay_handle_buffer (GstRTPBasePayload * basepayload,
       case JPEG_MARKER_SOS:
         sos_found = TRUE;
         GST_LOG_OBJECT (pay, "SOS found");
-        jpeg_header_size =
-            memory.offset + parse_mem_inc_offset_guint16 (&memory);
+        jpeg_header_size = memory.offset;
+        /* Do not re-combine into single statement with previous line! */
+        jpeg_header_size += parse_mem_inc_offset_guint16 (&memory);
         break;
       case JPEG_MARKER_EOI:
         GST_WARNING_OBJECT (pay, "EOI reached before SOS!");
@@ -885,7 +889,9 @@ gst_rtp_jpeg_pay_handle_buffer (GstRTPBasePayload * basepayload,
     if (dri_found)
       header_size += sizeof (restart_marker_header);
 
-    outbuf = gst_rtp_buffer_new_allocate (header_size, 0, 0);
+    outbuf =
+        gst_rtp_base_payload_allocate_output_buffer (basepayload, header_size,
+        0, 0);
 
     gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
 
@@ -893,6 +899,7 @@ gst_rtp_jpeg_pay_handle_buffer (GstRTPBasePayload * basepayload,
       GST_LOG_OBJECT (pay, "last packet of frame");
       frame_done = TRUE;
       gst_rtp_buffer_set_marker (&rtp, 1);
+      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_MARKER);
     }
 
     payload = gst_rtp_buffer_get_payload (&rtp);
@@ -923,10 +930,12 @@ gst_rtp_jpeg_pay_handle_buffer (GstRTPBasePayload * basepayload,
       for (i = 0; i < 2; i++) {
         guint qsize;
         guint qt;
+        gsize rc GST_UNUSED_ASSERT;
 
         qt = info[i].qt;
         qsize = tables[qt].size;
-        memcpy (payload, tables[qt].data, qsize);
+        rc = gst_buffer_extract (buffer, tables[qt].offset, payload, qsize);
+        g_assert (rc == qsize);
 
         GST_LOG_OBJECT (pay, "component %d using quant %d, size %d", i, qt,
             qsize);
@@ -1045,10 +1054,3 @@ gst_rtp_jpeg_pay_get_property (GObject * object, guint prop_id,
       break;
   }
 }
-
-gboolean
-gst_rtp_jpeg_pay_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "rtpjpegpay", GST_RANK_SECONDARY,
-      GST_TYPE_RTP_JPEG_PAY);
-}
