@@ -70,6 +70,8 @@
 GST_DEBUG_CATEGORY_STATIC (videocrop_debug);
 #define GST_CAT_DEFAULT videocrop_debug
 
+static GQuark _size_quark;
+
 enum
 {
   PROP_0,
@@ -112,6 +114,8 @@ static gboolean gst_video_crop_set_info (GstVideoFilter * vfilter, GstCaps * in,
     GstVideoInfo * in_info, GstCaps * out, GstVideoInfo * out_info);
 static GstFlowReturn gst_video_crop_transform_frame (GstVideoFilter * vfilter,
     GstVideoFrame * in_frame, GstVideoFrame * out_frame);
+static gboolean gst_video_crop_transform_meta (GstBaseTransform * trans,
+    GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf);
 
 static gboolean gst_video_crop_decide_allocation (GstBaseTransform * trans,
     GstQuery * query);
@@ -123,48 +127,32 @@ static GstFlowReturn gst_video_crop_transform_ip (GstBaseTransform * trans,
 static gboolean
 gst_video_crop_src_event (GstBaseTransform * trans, GstEvent * event)
 {
-  GstEvent *new_event;
-  GstStructure *new_structure;
-  const GstStructure *structure;
-  const gchar *event_name;
-  double pointer_x;
-  double pointer_y;
+  double x, y, new_x, new_y;
 
   GstVideoCrop *vcrop = GST_VIDEO_CROP (trans);
-  new_event = NULL;
-
   GST_OBJECT_LOCK (vcrop);
-  if (GST_EVENT_TYPE (event) == GST_EVENT_NAVIGATION &&
-      (vcrop->crop_left != 0 || vcrop->crop_top != 0)) {
-    structure = gst_event_get_structure (event);
-    event_name = gst_structure_get_string (structure, "event");
 
-    if (event_name &&
-        (strcmp (event_name, "mouse-move") == 0 ||
-            strcmp (event_name, "mouse-button-press") == 0 ||
-            strcmp (event_name, "mouse-button-release") == 0)) {
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_NAVIGATION:
+      if ((vcrop->crop_left != 0 || vcrop->crop_top != 0)
+          && gst_navigation_event_get_coordinates (event, &x, &y)) {
 
-      if (gst_structure_get_double (structure, "pointer_x", &pointer_x) &&
-          gst_structure_get_double (structure, "pointer_y", &pointer_y)) {
+        new_x = x + vcrop->crop_left;
+        new_y = y + vcrop->crop_top;
 
-        new_structure = gst_structure_copy (structure);
-        gst_structure_set (new_structure,
-            "pointer_x", G_TYPE_DOUBLE, (double) (pointer_x + vcrop->crop_left),
-            "pointer_y", G_TYPE_DOUBLE, (double) (pointer_y + vcrop->crop_top),
-            NULL);
+        event = gst_event_make_writable (event);
 
-        new_event = gst_event_new_navigation (new_structure);
-        gst_event_unref (event);
-      } else {
-        GST_WARNING_OBJECT (vcrop, "Failed to read navigation event");
+        GST_TRACE_OBJECT (vcrop, "from %fx%f to %fx%f", x, y, new_x, new_y);
+        gst_navigation_event_set_coordinates (event, new_x, new_y);
       }
-    }
+      break;
+    default:
+      break;
   }
 
   GST_OBJECT_UNLOCK (vcrop);
 
-  return GST_BASE_TRANSFORM_CLASS (parent_class)->src_event (trans,
-      (new_event ? new_event : event));
+  return GST_BASE_TRANSFORM_CLASS (parent_class)->src_event (trans, event);
 }
 
 static void
@@ -211,12 +199,16 @@ gst_video_crop_class_init (GstVideoCropClass * klass)
       "Crops video into a user-defined region",
       "Tim-Philipp Müller <tim centricular net>");
 
+  _size_quark = g_quark_from_static_string (GST_META_TAG_VIDEO_SIZE_STR);
+
   basetransform_class->before_transform =
       GST_DEBUG_FUNCPTR (gst_video_crop_before_transform);
   basetransform_class->transform_ip_on_passthrough = FALSE;
   basetransform_class->transform_caps =
       GST_DEBUG_FUNCPTR (gst_video_crop_transform_caps);
   basetransform_class->src_event = GST_DEBUG_FUNCPTR (gst_video_crop_src_event);
+  basetransform_class->transform_meta = gst_video_crop_transform_meta;
+
   basetransform_class->decide_allocation =
       GST_DEBUG_FUNCPTR (gst_video_crop_decide_allocation);
   basetransform_class->propose_allocation =
@@ -240,10 +232,8 @@ gst_video_crop_init (GstVideoCrop * vcrop)
   vcrop->crop_bottom = 0;
 }
 
-#define ROUND_DOWN_2(n)  ((n)&(~1))
-
 static void
-gst_video_crop_transform_packed_complex (GstVideoCrop * vcrop,
+gst_video_crop_transform_packed_yvyu (GstVideoCrop * vcrop,
     GstVideoFrame * in_frame, GstVideoFrame * out_frame, gint x, gint y)
 {
   guint8 *in_data, *out_data;
@@ -265,7 +255,7 @@ gst_video_crop_transform_packed_complex (GstVideoCrop * vcrop,
 
   /* rounding down here so we end up at the start of a macro-pixel and not
    * in the middle of one */
-  in_data += ROUND_DOWN_2 (vcrop->crop_left) *
+  in_data += GST_ROUND_DOWN_2 (vcrop->crop_left) *
       GST_VIDEO_FRAME_COMP_PSTRIDE (in_frame, 0);
 
   dx = width * GST_VIDEO_FRAME_COMP_PSTRIDE (out_frame, 0);
@@ -295,6 +285,41 @@ gst_video_crop_transform_packed_complex (GstVideoCrop * vcrop,
   }
 }
 
+static void
+gst_video_crop_transform_packed_v210 (GstVideoCrop * vcrop,
+    GstVideoFrame * in_frame, GstVideoFrame * out_frame, gint x, gint y)
+{
+  guint8 *in_data, *out_data;
+  guint i, dx;
+  gint width, height;
+  gint in_stride;
+  gint out_stride;
+
+  width = GST_VIDEO_FRAME_WIDTH (out_frame);
+  height = GST_VIDEO_FRAME_HEIGHT (out_frame);
+
+  in_data = GST_VIDEO_FRAME_PLANE_DATA (in_frame, 0);
+  out_data = GST_VIDEO_FRAME_PLANE_DATA (out_frame, 0);
+
+  in_stride = GST_VIDEO_FRAME_PLANE_STRIDE (in_frame, 0);
+  out_stride = GST_VIDEO_FRAME_PLANE_STRIDE (out_frame, 0);
+
+  in_data += vcrop->crop_top * in_stride;
+
+  /* rounding down here so we end up at the start of a macro-pixel and not
+   * in the middle of one */
+  in_data += (vcrop->crop_left / 6) * 16;
+
+  /* copy a whole set of macro-pixels */
+  dx = ((width + 5) / 6) * 16;
+
+  for (i = 0; i < height; ++i) {
+    memcpy (out_data, in_data, dx);
+    in_data += in_stride;
+    out_data += out_stride;
+  }
+}
+
 static void
 gst_video_crop_transform_packed_simple (GstVideoCrop * vcrop,
     GstVideoFrame * in_frame, GstVideoFrame * out_frame, gint x, gint y)
@@ -448,9 +473,11 @@ gst_video_crop_transform_frame (GstVideoFilter * vfilter,
     case VIDEO_CROP_PIXEL_FORMAT_PACKED_SIMPLE:
       gst_video_crop_transform_packed_simple (vcrop, in_frame, out_frame, x, y);
       break;
-    case VIDEO_CROP_PIXEL_FORMAT_PACKED_COMPLEX:
-      gst_video_crop_transform_packed_complex (vcrop, in_frame, out_frame, x,
-          y);
+    case VIDEO_CROP_PIXEL_FORMAT_PACKED_YVYU:
+      gst_video_crop_transform_packed_yvyu (vcrop, in_frame, out_frame, x, y);
+      break;
+    case VIDEO_CROP_PIXEL_FORMAT_PACKED_v210:
+      gst_video_crop_transform_packed_v210 (vcrop, in_frame, out_frame, x, y);
       break;
     case VIDEO_CROP_PIXEL_FORMAT_PLANAR:
       gst_video_crop_transform_planar (vcrop, in_frame, out_frame, x, y);
@@ -833,7 +860,7 @@ gst_video_crop_set_info (GstVideoFilter * vfilter, GstCaps * in,
     case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_YUY2:
     case GST_VIDEO_FORMAT_UYVY:
-      crop->packing = VIDEO_CROP_PIXEL_FORMAT_PACKED_COMPLEX;
+      crop->packing = VIDEO_CROP_PIXEL_FORMAT_PACKED_YVYU;
       if (GST_VIDEO_INFO_FORMAT (in_info) == GST_VIDEO_FORMAT_UYVY) {
         /* UYVY = 4:2:2 - [U0 Y0 V0 Y1] [U2 Y2 V2 Y3] [U4 Y4 V4 Y5] */
         crop->macro_y_off = 1;
@@ -842,6 +869,9 @@ gst_video_crop_set_info (GstVideoFilter * vfilter, GstCaps * in,
         crop->macro_y_off = 0;
       }
       break;
+    case GST_VIDEO_FORMAT_v210:
+      crop->packing = VIDEO_CROP_PIXEL_FORMAT_PACKED_v210;
+      break;
     case GST_VIDEO_FORMAT_I420:
     case GST_VIDEO_FORMAT_I420_10BE:
     case GST_VIDEO_FORMAT_I420_10LE:
@@ -921,6 +951,71 @@ gst_video_crop_set_crop (GstVideoCrop * vcrop, gint new_value, gint * prop)
   }
 }
 
+static gboolean
+gst_video_crop_transform_meta (GstBaseTransform * trans,
+    GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf)
+{
+  GstVideoCrop *video_crop = GST_VIDEO_CROP (trans);
+  const GstMetaInfo *info = meta->info;
+  const gchar *const *tags;
+  const gchar *const *curr = NULL;
+  gboolean should_copy = TRUE;
+  const gchar *const valid_tags[] = {
+    GST_META_TAG_VIDEO_STR,
+    GST_META_TAG_VIDEO_ORIENTATION_STR,
+    GST_META_TAG_VIDEO_SIZE_STR,
+    NULL
+  };
+
+  tags = gst_meta_api_type_get_tags (info->api);
+
+  /* No specific tags, we are good to copy */
+  if (!tags) {
+    return TRUE;
+  }
+
+  /* We are only changing size, we can preserve other metas tagged as
+     orientation and colorspace */
+  for (curr = tags; *curr; ++curr) {
+    /* We dont handle any other tag */
+    if (!g_strv_contains (valid_tags, *curr)) {
+      should_copy = FALSE;
+      break;
+    }
+  }
+  /* Cant handle the tags in this meta, let the parent class handle it */
+  if (!should_copy) {
+    return GST_BASE_TRANSFORM_CLASS (parent_class)->transform_meta (trans,
+        outbuf, meta, inbuf);
+  }
+
+  /* This meta is size sensitive, try to transform it accordingly */
+  if (gst_meta_api_type_has_tag (info->api, _size_quark)) {
+    if (info->transform_func) {
+      GstVideoMetaTransformMatrix trans_matrix;
+      const GstVideoRectangle in_rectangle = { video_crop->crop_left,
+        video_crop->crop_top, GST_VIDEO_INFO_WIDTH (&video_crop->out_info),
+        GST_VIDEO_INFO_HEIGHT (&video_crop->out_info)
+      };
+      const GstVideoRectangle out_rectangle = { 0, 0,
+        GST_VIDEO_INFO_WIDTH (&video_crop->out_info),
+        GST_VIDEO_INFO_HEIGHT (&video_crop->out_info)
+      };
+
+      gst_video_meta_transform_matrix_init (&trans_matrix,
+          &video_crop->in_info, &in_rectangle,
+          &video_crop->out_info, &out_rectangle);
+
+      info->transform_func (outbuf, meta, inbuf,
+          gst_video_meta_transform_matrix_get_quark (), &trans_matrix);
+    }
+    return FALSE;
+  }
+
+  /* No need to transform, we can safely copy this meta */
+  return TRUE;
+}
+
 static void
 gst_video_crop_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
