@@ -76,9 +76,13 @@ enum
 enum
 {
   ARG_0,
-  ARG_DEVICE
+  ARG_DEVICE,
+  ARG_UNIQUE_ID,
+  ARG_CONFIGURE_SESSION,
 };
 
+#define DEFAULT_CONFIGURE_SESSION TRUE
+
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -89,6 +93,7 @@ static void gst_osx_audio_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_osx_audio_src_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
+static void gst_osx_audio_src_finalize (GObject * object);
 
 static GstStateChangeReturn
 gst_osx_audio_src_change_state (GstElement * element,
@@ -141,6 +146,7 @@ gst_osx_audio_src_class_init (GstOsxAudioSrcClass * klass)
 
   gobject_class->set_property = gst_osx_audio_src_set_property;
   gobject_class->get_property = gst_osx_audio_src_get_property;
+  gobject_class->finalize = gst_osx_audio_src_finalize;
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_osx_audio_src_change_state);
@@ -151,6 +157,39 @@ gst_osx_audio_src_class_init (GstOsxAudioSrcClass * klass)
       g_param_spec_int ("device", "Device ID", "Device ID of input device",
           0, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * osxaudiosrc:unique-id
+   *
+   * Unique persistent ID for the input device
+   *
+   * Since: 1.26
+   */
+  g_object_class_install_property (gobject_class, ARG_UNIQUE_ID,
+      g_param_spec_string ("unique-id", "Unique ID",
+          "Unique persistent ID for the input device",
+          NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+#ifdef HAVE_IOS
+  /**
+   * GstOsxAudioSrc:configure-session:
+   *
+   * Whether the app-wide AVAudioSession should be automatically set up for audio capture.
+   * This will set the category to AVAudioSessionCategoryPlayAndRecord and activate
+   * the session when the element goes to READY. No other settings will be changed.
+   *
+   * If your application needs to configure anything more than the category, set this to FALSE
+   * for all osxaudiosink/src instances and handle the AVAudioSession setup yourself.
+   *
+   * Since: 1.26
+   */
+  g_object_class_install_property (gobject_class, ARG_CONFIGURE_SESSION,
+      g_param_spec_boolean ("configure-session",
+          "Enable automatic AVAudioSession setup",
+          "Whether the app-wide AVAudioSession should be automatically configured for audio capture",
+          DEFAULT_CONFIGURE_SESSION,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+#endif
+
   gstaudiobasesrc_class->create_ringbuffer =
       GST_DEBUG_FUNCPTR (gst_osx_audio_src_create_ringbuffer);
 
@@ -168,6 +207,19 @@ gst_osx_audio_src_init (GstOsxAudioSrc * src)
   gst_base_src_set_live (GST_BASE_SRC (src), TRUE);
 
   src->device_id = kAudioDeviceUnknown;
+  src->unique_id = NULL;
+
+#ifdef HAVE_IOS
+  src->configure_session = DEFAULT_CONFIGURE_SESSION;
+#endif
+}
+
+static void
+gst_osx_audio_src_finalize (GObject * object)
+{
+  GstOsxAudioSrc *src = GST_OSX_AUDIO_SRC (object);
+  g_clear_pointer (&src->unique_id, g_free);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -180,6 +232,14 @@ gst_osx_audio_src_set_property (GObject * object, guint prop_id,
     case ARG_DEVICE:
       src->device_id = g_value_get_int (value);
       break;
+    case ARG_UNIQUE_ID:
+      src->unique_id = g_value_dup_string (value);
+      break;
+#ifdef HAVE_IOS
+    case ARG_CONFIGURE_SESSION:
+      src->configure_session = g_value_get_boolean (value);
+      break;
+#endif
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -196,6 +256,14 @@ gst_osx_audio_src_get_property (GObject * object, guint prop_id,
     case ARG_DEVICE:
       g_value_set_int (value, src->device_id);
       break;
+    case ARG_UNIQUE_ID:
+      g_value_set_string (value, src->unique_id);
+      break;
+#ifdef HAVE_IOS
+    case ARG_CONFIGURE_SESSION:
+      g_value_set_boolean (value, src->configure_session);
+      break;
+#endif
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -210,6 +278,20 @@ gst_osx_audio_src_change_state (GstElement * element, GstStateChange transition)
   GstStateChangeReturn ret;
 
   switch (transition) {
+    case GST_STATE_CHANGE_READY_TO_NULL:{
+      GST_OBJECT_LOCK (osxsrc);
+      osxsrc->device_id = kAudioDeviceUnknown;
+      g_clear_pointer (&osxsrc->unique_id, g_free);
+      GST_OBJECT_UNLOCK (osxsrc);
+      break;
+    }
+#ifdef HAVE_IOS
+    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
+      ringbuffer =
+          GST_OSX_AUDIO_RING_BUFFER (GST_AUDIO_BASE_SRC (osxsrc)->ringbuffer);
+      ringbuffer->core_audio->first_sample_time = -1;
+      break;
+#endif
     default:
       break;
   }
@@ -224,7 +306,12 @@ gst_osx_audio_src_change_state (GstElement * element, GstStateChange transition)
       ringbuffer =
           GST_OSX_AUDIO_RING_BUFFER (GST_AUDIO_BASE_SRC (osxsrc)->ringbuffer);
       if (ringbuffer->core_audio->device_id != osxsrc->device_id) {
+        GST_OBJECT_LOCK (osxsrc);
         osxsrc->device_id = ringbuffer->core_audio->device_id;
+        g_free (osxsrc->unique_id);
+        osxsrc->unique_id = g_strdup (ringbuffer->core_audio->unique_id);
+        GST_OBJECT_UNLOCK (osxsrc);
+
         g_object_notify (G_OBJECT (osxsrc), "device");
       }
       break;
@@ -314,16 +401,15 @@ gst_osx_audio_src_create_ringbuffer (GstAudioBaseSrc * src)
       GST_OSX_AUDIO_ELEMENT_GET_INTERFACE (osxsrc),
       (void *) gst_osx_audio_src_io_proc);
 
+  ringbuffer->core_audio = g_object_new (GST_TYPE_CORE_AUDIO, "is-src", TRUE,
+      "device", osxsrc->device_id, "unique-id", osxsrc->unique_id,
+#ifdef HAVE_IOS
+      "configure-session", osxsrc->configure_session,
+#endif
+      NULL);
+  ringbuffer->core_audio->osxbuf = GST_OBJECT (ringbuffer);
   ringbuffer->core_audio->element =
       GST_OSX_AUDIO_ELEMENT_GET_INTERFACE (osxsrc);
-  ringbuffer->core_audio->is_src = TRUE;
-
-  /* By default the coreaudio instance created by the ringbuffer
-   * has device_id==kAudioDeviceUnknown. The user might have
-   * selected a different one here
-   */
-  if (ringbuffer->core_audio->device_id != osxsrc->device_id)
-    ringbuffer->core_audio->device_id = osxsrc->device_id;
 
   return GST_AUDIO_RING_BUFFER (ringbuffer);
 }
@@ -369,6 +455,22 @@ gst_osx_audio_src_io_proc (GstOsxAudioRingBuffer * buf,
   remaining = buf->core_audio->recBufferList->mBuffers[0].mDataByteSize;
   sample_position = inTimeStamp->mSampleTime;
 
+#ifdef HAVE_IOS
+  /* Timestamps don't always start from 0 on iOS, have to offset */
+  if (buf->core_audio->first_sample_time == -1) {
+    GST_DEBUG ("Setting first CoreAudio timestamp to %f",
+        inTimeStamp->mSampleTime);
+    buf->core_audio->first_sample_time = inTimeStamp->mSampleTime;
+  }
+
+  sample_position -= buf->core_audio->first_sample_time;
+#endif
+
+  if (g_atomic_int_get (&buf->core_audio->io_proc_dropping)) {
+    GST_TRACE_OBJECT (buf, "Dropped %d samples", remaining);
+    return 0;
+  }
+
   while (remaining) {
     if (!gst_audio_ring_buffer_prepare_read (GST_AUDIO_RING_BUFFER (buf),
             &writeseg, &writeptr, &len))
