@@ -73,7 +73,7 @@ struct _GstOsxAudioSrc
   GstAudioBaseSrc src;
 
   AudioDeviceID device_id;
-  const char *unique_id;
+  char *unique_id;
 
 #ifdef HAVE_IOS
   gboolean configure_session;
