@@ -23,19 +23,26 @@
 
 #include "gstosxcoreaudiocommon.h"
 
+GST_DEBUG_CATEGORY_EXTERN (osx_coreaudio_debug);
+#define GST_CAT_DEFAULT osx_coreaudio_debug
+
 void
 gst_core_audio_remove_render_callback (GstCoreAudio * core_audio)
 {
   AURenderCallbackStruct input;
   OSStatus status;
+  AudioUnitPropertyID callback_type;
 
   /* Deactivate the render callback by calling SetRenderCallback
    * with a NULL inputProc.
    */
   input.inputProc = NULL;
   input.inputProcRefCon = NULL;
+  callback_type = core_audio->is_src ?
+      kAudioOutputUnitProperty_SetInputCallback :
+      kAudioUnitProperty_SetRenderCallback;
 
-  status = AudioUnitSetProperty (core_audio->audiounit, kAudioUnitProperty_SetRenderCallback, kAudioUnitScope_Global, 0,        /* N/A for global */
+  status = AudioUnitSetProperty (core_audio->audiounit, callback_type, kAudioUnitScope_Global, 0,       /* N/A for global */
       &input, sizeof (input));
 
   if (status) {
@@ -43,42 +50,11 @@ gst_core_audio_remove_render_callback (GstCoreAudio * core_audio)
         "Failed to remove render callback %d", (int) status);
   }
 
-  /* Remove the RenderNotify too */
-  status = AudioUnitRemoveRenderNotify (core_audio->audiounit,
-      (AURenderCallback) gst_core_audio_render_notify, core_audio);
-
-  if (status) {
-    GST_WARNING_OBJECT (core_audio->osxbuf,
-        "Failed to remove render notify callback %d", (int) status);
-  }
-
   /* We're deactivated.. */
-  core_audio->io_proc_needs_deactivation = FALSE;
+  g_atomic_int_set (&core_audio->io_proc_dropping, FALSE);
   core_audio->io_proc_active = FALSE;
 }
 
-OSStatus
-gst_core_audio_render_notify (GstCoreAudio * core_audio,
-    AudioUnitRenderActionFlags * ioActionFlags,
-    const AudioTimeStamp * inTimeStamp,
-    unsigned int inBusNumber,
-    unsigned int inNumberFrames, AudioBufferList * ioData)
-{
-  /* Before rendering a frame, we get the PreRender notification.
-   * Here, we detach the RenderCallback if we've been paused.
-   *
-   * This is necessary (rather than just directly detaching it) to
-   * work around some thread-safety issues in CoreAudio
-   */
-  if ((*ioActionFlags) & kAudioUnitRenderAction_PreRender) {
-    if (core_audio->io_proc_needs_deactivation) {
-      gst_core_audio_remove_render_callback (core_audio);
-    }
-  }
-
-  return noErr;
-}
-
 gboolean
 gst_core_audio_io_proc_start (GstCoreAudio * core_audio)
 {
@@ -105,21 +81,17 @@ gst_core_audio_io_proc_start (GstCoreAudio * core_audio)
           "AudioUnitSetProperty failed: %d", (int) status);
       return FALSE;
     }
-    // ### does it make sense to do this notify stuff for input mode?
-    status = AudioUnitAddRenderNotify (core_audio->audiounit,
-        (AURenderCallback) gst_core_audio_render_notify, core_audio);
 
-    if (status) {
-      GST_ERROR_OBJECT (core_audio->osxbuf,
-          "AudioUnitAddRenderNotify failed %d", (int) status);
-      return FALSE;
-    }
     core_audio->io_proc_active = TRUE;
   }
 
-  core_audio->io_proc_needs_deactivation = FALSE;
+  g_atomic_int_set (&core_audio->io_proc_dropping, FALSE);
 
+  // AudioOutputUnitStart on iOS can wait for the render callback to finish,
+  // where in our case we set the ringbuffer timestamp, which also needs the ringbuf lock.
+  GST_OBJECT_UNLOCK (core_audio->osxbuf);
   status = AudioOutputUnitStart (core_audio->audiounit);
+  GST_OBJECT_LOCK (core_audio->osxbuf);
   if (status) {
     GST_ERROR_OBJECT (core_audio->osxbuf, "AudioOutputUnitStart failed: %d",
         (int) status);
@@ -142,7 +114,7 @@ gst_core_audio_io_proc_stop (GstCoreAudio * core_audio)
     GST_WARNING_OBJECT (core_audio->osxbuf,
         "AudioOutputUnitStop failed: %d", (int) status);
   }
-  // ###: why is it okay to directly remove from here but not from pause() ?
+  // Ok to remove directly here because we stopped the AudioUnit already
   if (core_audio->io_proc_active) {
     gst_core_audio_remove_render_callback (core_audio);
   }
@@ -194,8 +166,8 @@ gst_core_audio_bind_device (GstCoreAudio * core_audio)
   OSStatus status;
 
   /* Specify which device we're using. */
-  GST_DEBUG_OBJECT (core_audio->osxbuf, "Bind AudioUnit to device %d",
-      (int) core_audio->device_id);
+  GST_DEBUG_OBJECT (core_audio->osxbuf, "Bind AudioUnit to device %s",
+      core_audio->unique_id);
   status = AudioUnitSetProperty (core_audio->audiounit,
       kAudioOutputUnitProperty_CurrentDevice, kAudioUnitScope_Global, 0,
       &core_audio->device_id, sizeof (AudioDeviceID));
@@ -385,9 +357,9 @@ gst_audio_channel_position_to_core_audio (GstAudioChannelPosition
     case GST_AUDIO_CHANNEL_POSITION_REAR_CENTER:
       return kAudioChannelLabel_CenterSurround;
     case GST_AUDIO_CHANNEL_POSITION_REAR_LEFT:
-      return kAudioChannelLabel_LeftSurround;
+      return kAudioChannelLabel_RearSurroundLeft;
     case GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT:
-      return kAudioChannelLabel_RightSurround;
+      return kAudioChannelLabel_RearSurroundRight;
     case GST_AUDIO_CHANNEL_POSITION_LFE1:
       return kAudioChannelLabel_LFEScreen;
     case GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER:
@@ -418,6 +390,12 @@ gst_audio_channel_position_to_core_audio (GstAudioChannelPosition
       return kAudioChannelLabel_VerticalHeightRight;
     case GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_CENTER:
       return kAudioChannelLabel_VerticalHeightCenter;
+    case GST_AUDIO_CHANNEL_POSITION_TOP_CENTER:
+      return kAudioChannelLabel_TopCenterSurround;
+    case GST_AUDIO_CHANNEL_POSITION_SURROUND_LEFT:
+      return kAudioChannelLabel_LeftSurround;
+    case GST_AUDIO_CHANNEL_POSITION_SURROUND_RIGHT:
+      return kAudioChannelLabel_RightSurround;
 
       /* Special position values */
     case GST_AUDIO_CHANNEL_POSITION_NONE:
@@ -427,14 +405,11 @@ gst_audio_channel_position_to_core_audio (GstAudioChannelPosition
 
       /* Following positions are unmapped --
        * i.e. mapped to kAudioChannelLabel_Unknown: */
-    case GST_AUDIO_CHANNEL_POSITION_TOP_CENTER:
     case GST_AUDIO_CHANNEL_POSITION_TOP_SIDE_LEFT:
     case GST_AUDIO_CHANNEL_POSITION_TOP_SIDE_RIGHT:
     case GST_AUDIO_CHANNEL_POSITION_BOTTOM_FRONT_CENTER:
     case GST_AUDIO_CHANNEL_POSITION_BOTTOM_FRONT_LEFT:
     case GST_AUDIO_CHANNEL_POSITION_BOTTOM_FRONT_RIGHT:
-    case GST_AUDIO_CHANNEL_POSITION_SURROUND_LEFT:
-    case GST_AUDIO_CHANNEL_POSITION_SURROUND_RIGHT:
     default:
       return kAudioChannelLabel_Unknown;
   }
@@ -443,7 +418,7 @@ gst_audio_channel_position_to_core_audio (GstAudioChannelPosition
 /* Performs a best-effort conversion. 'channel' is used for warnings only. */
 GstAudioChannelPosition
 gst_core_audio_channel_label_to_gst (AudioChannelLabel label,
-    int channel, gboolean warn)
+    int channel, gboolean warn, AudioDeviceID device_id)
 {
   switch (label) {
     case kAudioChannelLabel_Left:
@@ -455,9 +430,9 @@ gst_core_audio_channel_label_to_gst (AudioChannelLabel label,
     case kAudioChannelLabel_LFEScreen:
       return GST_AUDIO_CHANNEL_POSITION_LFE1;
     case kAudioChannelLabel_LeftSurround:
-      return GST_AUDIO_CHANNEL_POSITION_REAR_LEFT;
+      return GST_AUDIO_CHANNEL_POSITION_SURROUND_LEFT;
     case kAudioChannelLabel_RightSurround:
-      return GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT;
+      return GST_AUDIO_CHANNEL_POSITION_SURROUND_RIGHT;
     case kAudioChannelLabel_LeftSurroundDirect:
       return GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT;
     case kAudioChannelLabel_RightSurroundDirect:
@@ -486,6 +461,12 @@ gst_core_audio_channel_label_to_gst (AudioChannelLabel label,
       return GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_RIGHT;
     case kAudioChannelLabel_VerticalHeightCenter:
       return GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_CENTER;
+    case kAudioChannelLabel_RearSurroundLeft:
+      return GST_AUDIO_CHANNEL_POSITION_REAR_LEFT;
+    case kAudioChannelLabel_RearSurroundRight:
+      return GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT;
+    case kAudioChannelLabel_TopCenterSurround:
+      return GST_AUDIO_CHANNEL_POSITION_TOP_CENTER;
 
       /* Special position values */
 
@@ -499,9 +480,6 @@ gst_core_audio_channel_label_to_gst (AudioChannelLabel label,
          Following labels are unmapped --
          i.e. mapped to GST_AUDIO_CHANNEL_POSITION_INVALID:
        */
-    case kAudioChannelLabel_RearSurroundLeft:
-    case kAudioChannelLabel_RearSurroundRight:
-    case kAudioChannelLabel_TopCenterSurround:
     case kAudioChannelLabel_LeftTotal:
     case kAudioChannelLabel_RightTotal:
     case kAudioChannelLabel_HearingImpaired:
@@ -514,14 +492,14 @@ gst_core_audio_channel_label_to_gst (AudioChannelLabel label,
         /* no way to store discrete channel order */
         if (warn)
           GST_WARNING
-              ("Core Audio channel %u labeled kAudioChannelLabel_Discrete_%u -- discrete order will be lost",
-              channel, ((unsigned int) label) & 0xFFFF);
+              ("Device %d has kAudioChannelLabel_Discrete_N channels, order will be lost",
+              device_id);
         return GST_AUDIO_CHANNEL_POSITION_NONE;
       } else {
         if (warn)
           GST_WARNING
-              ("Core Audio channel %u has unsupported label %d and will be skipped",
-              channel, (int) label);
+              ("Device %d channel %u has unsupported label %d, skipping",
+              device_id, channel, (int) label);
         return GST_AUDIO_CHANNEL_POSITION_INVALID;
       }
   }
@@ -550,3 +528,79 @@ gst_core_audio_dump_channel_layout (AudioChannelLayout * channel_layout)
         channel_desc->mCoordinates[2]);
   }
 }
+
+#ifndef HAVE_IOS
+char *
+gst_core_audio_device_get_prop_str (AudioDeviceID device_id,
+    AudioObjectPropertyElement prop_id)
+{
+  OSStatus status = noErr;
+  UInt32 propertySize = 0;
+  CFStringRef prop_val;
+  gchar *result = NULL;
+
+  AudioObjectPropertyAddress propAddress = {
+    prop_id,
+    kAudioDevicePropertyScopeOutput,
+    kAudioObjectPropertyElementMain
+  };
+
+  propAddress.mScope = kAudioObjectPropertyScopeGlobal;
+
+  /* Get the length of the device name */
+  status = AudioObjectGetPropertyDataSize (device_id,
+      &propAddress, 0, NULL, &propertySize);
+  if (status != noErr) {
+    goto beach;
+  }
+
+  /* Get the requested property */
+  status = AudioObjectGetPropertyData (device_id,
+      &propAddress, 0, NULL, &propertySize, &prop_val);
+  if (status != noErr) {
+    goto beach;
+  }
+
+  /* Convert to UTF-8 C String */
+  CFIndex prop_len = CFStringGetLength (prop_val);
+  CFIndex max_size =
+      CFStringGetMaximumSizeForEncoding (prop_len, kCFStringEncodingUTF8) + 1;
+  result = g_malloc (max_size);
+
+  if (!CFStringGetCString (prop_val, result, max_size, kCFStringEncodingUTF8)) {
+    g_free (result);
+    result = NULL;
+  }
+
+  CFRelease (prop_val);
+
+beach:
+  return result;
+}
+
+UInt32
+gst_core_audio_device_get_prop_uint32 (AudioDeviceID device_id,
+    AudioObjectPropertyElement prop_id)
+{
+  OSStatus status = noErr;
+  UInt32 propertySize = sizeof (UInt32);
+  UInt32 result = UINT32_MAX;
+
+  AudioObjectPropertyAddress propAddress = {
+    .mSelector = prop_id,
+    .mScope = kAudioObjectPropertyScopeGlobal,
+    .mElement = kAudioObjectPropertyElementMain,
+  };
+
+  /* Get the requested property */
+  status = AudioObjectGetPropertyData (device_id,
+      &propAddress, 0, NULL, &propertySize, &result);
+  if (status != noErr) {
+    GST_WARNING ("Device %i, error code %i while fetching property %"
+        GST_FOURCC_FORMAT, device_id, status,
+        GST_FOURCC_ARGS (GUINT32_FROM_BE (prop_id)));
+  }
+
+  return result;
+}
+#endif
