@@ -80,6 +80,15 @@ enum _GstV4l2BufferState
   BUFFER_STATE_QUEUED = 2,
 };
 
+enum
+{
+  OUTPUT_ERROR_DEQUEUED,
+  CAPTURE_ERROR_DEQUEUED,
+  LAST_SIGNAL
+};
+
+static guint gst_v4l2_buffer_pool_signals[LAST_SIGNAL] = { 0 };
+
 static void gst_v4l2_buffer_pool_complete_release_buffer (GstBufferPool * bpool,
     GstBuffer * buffer, gboolean queued);
 
@@ -139,7 +148,7 @@ static GstFlowReturn
 gst_v4l2_buffer_pool_copy_buffer (GstV4l2BufferPool * pool, GstBuffer * dest,
     GstBuffer * src)
 {
-  const GstVideoFormatInfo *finfo = pool->caps_info.finfo;
+  const GstVideoFormatInfo *finfo = pool->caps_info.vinfo.finfo;
 
   GST_LOG_OBJECT (pool, "copying buffer");
 
@@ -150,10 +159,11 @@ gst_v4l2_buffer_pool_copy_buffer (GstV4l2BufferPool * pool, GstBuffer * dest,
     GST_DEBUG_OBJECT (pool, "copy video frame");
 
     /* we have raw video, use videoframe copy to get strides right */
-    if (!gst_video_frame_map (&src_frame, &pool->caps_info, src, GST_MAP_READ))
+    if (!gst_video_frame_map (&src_frame, &pool->caps_info.vinfo, src,
+            GST_MAP_READ))
       goto invalid_buffer;
 
-    if (!gst_video_frame_map (&dest_frame, &pool->caps_info, dest,
+    if (!gst_video_frame_map (&dest_frame, &pool->caps_info.vinfo, dest,
             GST_MAP_WRITE)) {
       gst_video_frame_unmap (&src_frame);
       goto invalid_buffer;
@@ -233,7 +243,7 @@ gst_v4l2_buffer_pool_import_userptr (GstV4l2BufferPool * pool,
   GstFlowReturn ret = GST_FLOW_OK;
   GstV4l2MemoryGroup *group = NULL;
   GstMapFlags flags;
-  const GstVideoFormatInfo *finfo = pool->caps_info.finfo;
+  const GstVideoFormatInfo *finfo = pool->caps_info.vinfo.finfo;
   struct UserPtrData *data = NULL;
 
   GST_LOG_OBJECT (pool, "importing userptr");
@@ -256,7 +266,7 @@ gst_v4l2_buffer_pool_import_userptr (GstV4l2BufferPool * pool,
 
     data->is_frame = TRUE;
 
-    if (!gst_video_frame_map (&data->frame, &pool->caps_info, src, flags))
+    if (!gst_video_frame_map (&data->frame, &pool->caps_info.vinfo, src, flags))
       goto invalid_buffer;
 
     for (i = 0; i < GST_VIDEO_FORMAT_INFO_N_PLANES (finfo); i++) {
@@ -630,7 +640,11 @@ gst_v4l2_buffer_pool_set_config (GstBufferPool * bpool, GstStructure * config)
       min_buffers, max_buffers);
 
   /* keep a GstVideoInfo with defaults for the when we need to copy */
-  gst_video_info_from_caps (&pool->caps_info, caps);
+  gst_video_info_dma_drm_init (&pool->caps_info);
+  if (!gst_video_info_dma_drm_from_caps (&pool->caps_info, caps) &&
+      !gst_video_info_from_caps (&pool->caps_info.vinfo, caps)) {
+    goto wrong_config;
+  }
 
 done:
   ret = GST_BUFFER_POOL_CLASS (parent_class)->set_config (bpool, config);
@@ -1279,11 +1293,21 @@ gst_v4l2_buffer_pool_dqbuf (GstV4l2BufferPool * pool, GstBuffer ** buffer,
   GST_LOG_OBJECT (pool, "dequeueing a buffer");
 
   res = gst_v4l2_allocator_dqbuf (pool->vallocator, &group);
+
   if (res == GST_V4L2_FLOW_LAST_BUFFER)
     goto eos;
   if (res != GST_FLOW_OK)
     goto dqbuf_failed;
 
+  if (group->buffer.flags & V4L2_BUF_FLAG_ERROR) {
+    if (V4L2_TYPE_IS_OUTPUT (obj->type))
+      g_signal_emit (pool, gst_v4l2_buffer_pool_signals[OUTPUT_ERROR_DEQUEUED],
+          0, (guint) group->buffer.timestamp.tv_sec);
+    else
+      g_signal_emit (pool, gst_v4l2_buffer_pool_signals[CAPTURE_ERROR_DEQUEUED],
+          0, (guint) group->buffer.timestamp.tv_sec);
+  }
+
   old_buffer_state =
       g_atomic_int_and (&pool->buffer_state[group->buffer.index],
       ~BUFFER_STATE_QUEUED);
@@ -1328,7 +1352,8 @@ gst_v4l2_buffer_pool_dqbuf (GstV4l2BufferPool * pool, GstBuffer ** buffer,
         group->planes[i].bytesused, i, group->buffer.flags,
         GST_TIME_ARGS (timestamp), pool->num_queued, outbuf, old_buffer_state);
 
-    if (GST_VIDEO_INFO_FORMAT (&pool->caps_info) == GST_VIDEO_FORMAT_ENCODED)
+    if (GST_VIDEO_INFO_FORMAT (&pool->caps_info.vinfo) ==
+        GST_VIDEO_FORMAT_ENCODED)
       break;
 
     if (obj->type == V4L2_BUF_TYPE_VIDEO_CAPTURE ||
@@ -1765,6 +1790,14 @@ gst_v4l2_buffer_pool_class_init (GstV4l2BufferPoolClass * klass)
   bufferpool_class->flush_start = gst_v4l2_buffer_pool_flush_start;
   bufferpool_class->flush_stop = gst_v4l2_buffer_pool_flush_stop;
 
+  gst_v4l2_buffer_pool_signals[OUTPUT_ERROR_DEQUEUED] =
+      g_signal_new ("output-error-dequeued", G_TYPE_FROM_CLASS (object_class),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_UINT);
+
+  gst_v4l2_buffer_pool_signals[CAPTURE_ERROR_DEQUEUED] =
+      g_signal_new ("capture-error-dequeued", G_TYPE_FROM_CLASS (object_class),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_UINT);
+
   GST_DEBUG_CATEGORY_INIT (v4l2bufferpool_debug, "v4l2bufferpool", 0,
       "V4L2 Buffer Pool");
   GST_DEBUG_CATEGORY_GET (CAT_PERFORMANCE, "GST_PERFORMANCE");
@@ -1948,7 +1981,7 @@ gst_v4l2_buffer_pool_process (GstV4l2BufferPool * pool, GstBuffer ** buf,
               goto eos;
             }
 
-            if (GST_VIDEO_INFO_FORMAT (&pool->caps_info) !=
+            if (GST_VIDEO_INFO_FORMAT (&pool->caps_info.vinfo) !=
                 GST_VIDEO_FORMAT_ENCODED && size < pool->size)
               goto buffer_truncated;
 
@@ -2158,7 +2191,7 @@ gst_v4l2_buffer_pool_process (GstV4l2BufferPool * pool, GstBuffer ** buf,
           }
 
           /* Save the amount of data that has been submitted for encoded data */
-          if (GST_VIDEO_INFO_FORMAT (&pool->caps_info) ==
+          if (GST_VIDEO_INFO_FORMAT (&pool->caps_info.vinfo) ==
               GST_VIDEO_FORMAT_ENCODED) {
             queued_size = gst_buffer_get_size (to_queue);
             remaining_size = gst_buffer_get_size (*buf) - queued_size;
