@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <gst/check/gstcheck.h>
@@ -209,7 +209,7 @@ got_buffer (GstElement * fakesink, GstBuffer * buf, GstPad * pad,
 
   gst_buffer_map (buf, &map, GST_MAP_READ);
 
-  GST_LOG ("got buffer, size=%u, offset=%" G_GINT64_FORMAT, map.size, off);
+  GST_LOG ("size=%" G_GSIZE_FORMAT ", offset=%" G_GINT64_FORMAT, map.size, off);
 
   fail_unless (GST_BUFFER_OFFSET_IS_VALID (buf));
 
@@ -242,7 +242,7 @@ test_taglib_apev2mux_check_output_buffer (GstBuffer * buf)
   guint off;
 
   gst_buffer_map (buf, &map, GST_MAP_READ);
-  g_assert (map.size % MP3_FRAME_SIZE == 0);
+  fail_unless_equals_uint64 (map.size % MP3_FRAME_SIZE, 0);
 
   for (off = 0; off < map.size; off += MP3_FRAME_SIZE) {
     fail_unless (memcmp (map.data + off, mp3_dummyhdr,
@@ -262,19 +262,19 @@ test_taglib_apev2mux_with_tags (GstTagList * tags, guint32 mask)
   GstBuffer *outbuf = NULL;
 
   pipeline = gst_pipeline_new ("pipeline");
-  g_assert (pipeline != NULL);
+  g_assert_nonnull (pipeline);
 
   fakesrc = gst_element_factory_make ("fakesrc", "fakesrc");
-  g_assert (fakesrc != NULL);
+  g_assert_nonnull (fakesrc);
 
   apev2mux = gst_element_factory_make ("apev2mux", "apev2mux");
-  g_assert (apev2mux != NULL);
+  g_assert_nonnull (apev2mux);
 
   apedemux = gst_element_factory_make ("apedemux", "apedemux");
-  g_assert (apedemux != NULL);
+  g_assert_nonnull (apedemux);
 
   fakesink = gst_element_factory_make ("fakesink", "fakesink");
-  g_assert (fakesink != NULL);
+  g_assert_nonnull (fakesink);
 
   /* set up sink */
   outbuf = NULL;
@@ -408,19 +408,4 @@ apev2mux_suite (void)
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = apev2mux_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (apev2mux);
