@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <gst/check/gstcheck.h>
@@ -294,7 +294,7 @@ got_buffer (GstElement * fakesink, GstBuffer * buf, GstPad * pad,
 
   gst_buffer_map (buf, &map, GST_MAP_READ);
 
-  GST_LOG ("got buffer, size=%u, offset=%" G_GINT64_FORMAT, map.size, off);
+  GST_LOG ("size=%" G_GSIZE_FORMAT ", offset=%" G_GINT64_FORMAT, map.size, off);
 
   fail_unless (GST_BUFFER_OFFSET_IS_VALID (buf));
 
@@ -328,7 +328,7 @@ test_taglib_id3mux_check_output_buffer (GstBuffer * buf)
   guint off;
 
   gst_buffer_map (buf, &map, GST_MAP_READ);
-  g_assert (map.size % MP3_FRAME_SIZE == 0);
+  fail_unless_equals_uint64 (map.size % MP3_FRAME_SIZE, 0);
 
   for (off = 0; off < map.size; off += MP3_FRAME_SIZE) {
     fail_unless (memcmp (map.data + off, mp3_dummyhdr,
@@ -358,22 +358,22 @@ test_taglib_id3mux_with_tags (GstTagList * tags, guint32 mask)
   GstStateChangeReturn state_result;
 
   pipeline = gst_pipeline_new ("pipeline");
-  g_assert (pipeline != NULL);
+  g_assert_nonnull (pipeline);
 
   fakesrc = gst_element_factory_make ("fakesrc", "fakesrc");
-  g_assert (fakesrc != NULL);
+  g_assert_nonnull (fakesrc);
 
   id3mux = gst_element_factory_make ("id3v2mux", "id3v2mux");
-  g_assert (id3mux != NULL);
+  g_assert_nonnull (id3mux);
 
   identity = gst_element_factory_make ("identity", "identity");
-  g_assert (identity != NULL);
+  g_assert_nonnull (identity);
 
   id3demux = gst_element_factory_make ("id3demux", "id3demux");
-  g_assert (id3demux != NULL);
+  g_assert_nonnull (id3demux);
 
   fakesink = gst_element_factory_make ("fakesink", "fakesink");
-  g_assert (fakesink != NULL);
+  g_assert_nonnull (fakesink);
 
   /* set up sink */
   outbuf = NULL;
@@ -518,19 +518,4 @@ id3v2mux_suite (void)
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = id3v2mux_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (id3v2mux);
