@@ -81,6 +81,7 @@ cleanup_jpegenc (GstElement * jpegenc)
   GST_DEBUG ("cleanup_jpegenc");
   gst_element_set_state (jpegenc, GST_STATE_NULL);
 
+  gst_check_drop_buffers ();
   gst_pad_set_active (mysrcpad, FALSE);
   gst_pad_set_active (mysinkpad, FALSE);
   gst_check_teardown_sink_pad (jpegenc);
@@ -101,7 +102,7 @@ create_video_buffer (GstCaps * caps)
       gst_parse_launch
       ("videotestsrc num-buffers=1 ! capsfilter name=cf ! appsink name=sink",
       NULL);
-  g_assert (pipeline != NULL);
+  g_assert_nonnull (pipeline);
 
   cf = gst_bin_get_by_name (GST_BIN (pipeline), "cf");
   sink = gst_bin_get_by_name (GST_BIN (pipeline), "sink");
