@@ -25,6 +25,12 @@
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/rtp/gstrtcpbuffer.h>
 
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+#define READ_UINT64(mem) GST_READ_UINT64_BE(mem)
+#else
+#define READ_UINT64(mem) GST_READ_UINT64_LE(mem)
+#endif
+
 /* UDP/IP is assumed for bandwidth calculation */
 #define UDP_IP_HEADER_OVERHEAD 28
 
@@ -105,6 +111,8 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 static GstBuffer *
 create_original_buffer (void)
 {
+  GstClock *clock;
+
   if (original_buffer != NULL)
     return original_buffer;
 
@@ -112,8 +120,9 @@ create_original_buffer (void)
       gst_buffer_new_wrapped ((guint8 *) payload, strlen (payload));
   fail_unless (original_buffer != NULL);
 
-  GST_BUFFER_TIMESTAMP (original_buffer) =
-      gst_clock_get_internal_time (gst_system_clock_obtain ());
+  clock = gst_system_clock_obtain ();
+  GST_BUFFER_TIMESTAMP (original_buffer) = gst_clock_get_internal_time (clock);
+  gst_object_unref (clock);
 
   return original_buffer;
 }
@@ -236,8 +245,8 @@ check_header (GstBuffer * buffer, guint index)
    * most likely be changed in gstrtpbin.
    */
   fail_unless (info.data != NULL);
-  fail_unless_equals_uint64 (*(guint64 *) info.data,
-      *(guint64 *) rtp_header[index]);
+  fail_unless_equals_uint64 (READ_UINT64 (info.data),
+      READ_UINT64 (rtp_header[index]));
   fail_unless (*(guint16 *) (info.data + 12) ==
       *(guint16 *) (rtp_header[index] + 12));
 
@@ -975,7 +984,7 @@ get_source_stats (GstElement * rtpsession,
   g_object_get (rtpsession, "stats", &stats, NULL);
   stats_arr =
       g_value_get_boxed (gst_structure_get_value (stats, "source-stats"));
-  g_assert (stats_arr != NULL);
+  g_assert_nonnull (stats_arr);
   fail_unless (stats_arr->n_values >= 1);
 
   *source_stats = NULL;
